{% comment %}
This template is deliberately in the root because django searches for `500.html`. Projects
can override this if needed with project specific templates which have higher priority.
{% endcomment %}
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <title>Server Error</title>
    <style type="text/css" nonce="{{ request.csp_nonce  }}">
:root {
  --background-color: #fff;
  --foreground-color: #333;
  --button-background-color: #fff;
  --button-border-color: #ccc;
}

* {
  box-sizing: border-box;
}

/* Vertically and horizontally center content */
body {
  background-color: var(--background-color);
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 14px;
  line-height: 1.42857143;
  color: var(--foreground-color);
  text-align: center;
  text-shadow: 0 1px 1px rgba(0,0,0,.3);

  width: 100vw;
  height: 100vh;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center;
  margin: 0;
  padding: 0 20px;
}

body > * {
  margin-left: auto;
  margin-right: auto;
}

h1 {
  font-size: 36px;
  font-family: inherit;
  font-weight: 500;
  line-height: 1.1;
  margin-top: 20px;
  margin-bottom: 10px;
  margin-inline-start: 0;
  margin-inline-end: 0;
}

p {
  font-size: 16px;
  margin: 0 0 10px;
}

p.lead {
  margin-bottom: 20px;
  font-weight: 300;
  line-height: 1.4;
}

.button {
  display: inline-block;
  margin: 0;
  padding: 10px 20px;

  background-color: var(--button-background-color);
  background-image: linear-gradient(to bottom, #fff 0, #e0e0e0 100%);
  background-repeat: repeat-x;

  box-shadow: inset 0 1px 0 rgb(255 255 255 / 15%), 0 1px 1px rgb(0 0 0 / 8%);
  border: 1px solid transparent;
  border-color: var(--button-border-color);
  border-radius: 6px;

  color: var(--foreground-color);
  font-size: 18px;
  font-weight: bold;
  line-height: 1.3333333;
  text-decoration: none;
  text-shadow: none;
}

.button:hover {
  background-color: #e0e0e0;
  background-position: 0 -15px;
  border-color: #adadad;
}

.button:active {
  box-shadow: inset 0 3px 5px rgb(0 0 0 / 13%);
}

.button:focus,
.button:focus-visible {
  background-color: #e0e0e0;
  background-position: 0 -15px;
  border-color: #8c8c8c;
  outline: thin dotted;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px;
}

@media (min-width: 768px) {
  body > * {
    max-width: 660px;
    width: 100%;
  }

  p.lead {
    font-size: 21px;
  }
}

@media (max-width: 768px) {
  body {
    justify-content: flex-start;
  }
}
    </style>

</head>

<body>
  <h1 >Sorry, a server error has occurred (500)</h1>
  <p class="lead">We have been automatically notified about the problem and will investigate as soon as possible.</p>
  {% if request.sentry.id %}
    <p>If you need assistance, you may reference this error as
    <strong>{{ request.sentry.id }}</strong>.</p>
  {% endif %}
  <p class="lead">
    <a href="/" class="button">Back to home</a>
  </p>
</body>

</html>
